package main

import (
    "context"
    "database/sql"
    "flag"
    "fmt"
    "time"

    _ "github.com/lib/pq"
)

type Result struct {
    host_name string
    sync_time time.Duration
}

var user string = "icinga"
var dbname string = "wds"

func waitForDb(dbhost string, local_start_ts time.Time, expected_db_ts time.Time, critical_timeout time.Duration, result_ch chan<- Result) {
    psqlInfo := fmt.Sprintf("user=%s dbname=%s host=%s",
                    user, dbname, dbhost)
    var result Result
    result.host_name = dbhost

    for {
        if time.Since(local_start_ts) > critical_timeout {
            break;
        }
        db, err := sql.Open("postgres", psqlInfo)
        err = db.Ping()
        if err == nil {
            row := db.QueryRow("select ts from check_postgres_cluster_timestamp")
            var db_ts time.Time
            row.Scan(&db_ts)
            if db_ts.Equal(expected_db_ts) {
                result.sync_time = time.Since(local_start_ts)
                result_ch <- result
                return
            }
        }
        time.Sleep(100 * time.Millisecond)
    }
    result.sync_time = time.Since(local_start_ts)
    result_ch <- result
}

func main() {
    default_warning_timeout, _ := time.ParseDuration("5s")
    warning_timeout := *flag.Duration(
                                "warning",
                                default_warning_timeout,
                                "warning timeout")
    default_critical_timeout, _ := time.ParseDuration("10s")
    critical_timeout := *flag.Duration(
                                "critical",
                                default_critical_timeout,
                                "critical timeout")
    flag.Parse()
    databases := flag.Args()
    master := databases[0]
    all := databases[1:]

    psqlInfo := fmt.Sprintf("user=%s dbname=%s host=%s",
                    user, dbname, master)
    db, _ := sql.Open("postgres", psqlInfo)

    // check if table exist and create it if necessary
    // wds=> create table check_postgres_cluster_timestamp (ts timestamptz);

    ctx := context.Background()

    tx, _ := db.BeginTx(ctx, &sql.TxOptions{Isolation: sql.LevelRepeatableRead})
    _, err := tx.Exec("delete from check_postgres_cluster_timestamp")
    if err != nil {
	tx.Rollback()
	tx, _ = db.BeginTx(ctx, &sql.TxOptions{Isolation: sql.LevelRepeatableRead})
	_, err := tx.Exec("create table check_postgres_cluster_timestamp(ts timestamptz)")
	if err != nil {
	    status := "CRITICAL"
	    info := err.Error()
	    fmt.Println(status, "-", info)
	    return
	}
    }
    row := tx.QueryRow("insert into check_postgres_cluster_timestamp values(now()) returning ts")
    var expected_db_ts time.Time
    row.Scan(&expected_db_ts)
    tx.Commit()

    local_start_ts := time.Now()

    result_ch := make(chan Result)

    for _, dbhost := range all {
        go waitForDb(dbhost, local_start_ts, expected_db_ts, critical_timeout, result_ch)
    }

    status := "OK"
    info := ""
    for range all {
        result := <-result_ch
        if result.sync_time > critical_timeout {
            status = "CRITICAL"
        } else if result.sync_time > warning_timeout {
            status = "WARNING"
        }
        info += fmt.Sprintf("%s: %s; ", result.host_name, result.sync_time.String())
    }

    fmt.Println(status, "-", info)
}
